import { Component, Input, EventEmitter, AfterContentInit, ContentChild, Output } from "@angular/core";
import { ContentChildren, QueryList, OnInit } from "@angular/core";
import { Observable } from "rxjs/Observable";
import { ApiFilterInterface } from "../../interfaces/api-filter.interface";
import { PaginationComponent } from "../pagination/pagination.component";
import { SearchComponent } from "../search/search.component";
import { CounterComponent } from "../counter/counter.component";
import { ListRowComponent } from "../list-row/list.row.component";
import { ListPreviewComponent } from "../list-preview/list.preview.component";
import { NotFoundComponent } from "../not-found/not.found.component";
import { NoResultsComponent } from "../no-results/no.results.component";
import { ClearFiltersComponent } from "../clear-filters/clear.filters.component";
import { ShowMoreComponent } from "../show-more/show.more.component";
import { ListFilterComponent } from "../list-filter/list.filter.component";
import { DateFilterComponent } from "../date-filter/date.filter.component";
import { Subscription } from "rxjs/Subscription";
/**
 * This component is a container for managing list view components together.
 * Its purpose is to manage and disperse data from a single location making management easier.
 *
 * Basic Example Usage
 * -------------
 *
 * Component TS
 *
 * @Component({
 *    selector: 'c-my-component',
 *    template: `
 *        <c-list-view #list [dataService]="userService">
 *          <c-pagination></c-pagination>
 *          <div *ngFor="let row of list.rows">
 *            {{row.firstName}}
 *          </div>
 *        </c-list-view>
 *    `,
 *
 * })
 * class MyComponent {
 *     constructor(public userService: UserService) {}
 * }
 *
 */
var ListViewComponent = /** @class */ (function () {
    function ListViewComponent() {
        this.changePageEmitter = new EventEmitter();
        this.searchTerm = new EventEmitter();
        this.filtersEmitter = new EventEmitter();
        this.filterEmitter = new EventEmitter();
        this.clearFiltersEmitter = new EventEmitter();
        this.listPreviewVisibleEmitter = new EventEmitter();
        this.activeRowEmitter = new EventEmitter();
        this.loadMoreEmitter = new EventEmitter();
        this.pageChangeScrollTop = false;
        this.handleRequestErrorEmitter = new EventEmitter();
        this.filters = [];
        this.currentPage = 1;
        this.loading = true;
        this.meta = {};
        this.rows = [];
        this.previewHidden = true;
        this.originalFilters = [];
        this.lastSearch = "";
        this.updating = false;
    }
    /**
     * @inheritdoc
     */
    /**
         * @inheritdoc
         */
    ListViewComponent.prototype.ngOnInit = /**
         * @inheritdoc
         */
    function () {
        var _this = this;
        this.filters = this.dataService.getFilters();
        // Fetch original filters for reset.
        this.originalFilters = JSON.parse(JSON.stringify(this.filters));
        this.dataSource = this.dataService.setParam("filters", this.filters).fetchAll();
        // Subscription to page change triggered by `pagination` component
        this.changePageEmitter.subscribe(function (page) {
            _this.updating = true;
            _this.dataSource = _this.dataService.setParam("page", page).fetchAll();
            _this.updateSubscriptions().then(function () {
                if (_this.pageChangeScrollTop) {
                    document.body.scrollTop = 0;
                }
            });
        });
        // Subscription to search term change triggered by `search` component
        this.searchTerm.subscribe(function (term) {
            // Record last search
            // Record last search
            _this.lastSearch = term;
            // Reset pagination back to first page
            // Reset pagination back to first page
            _this.dataService.setParam("page", 1);
            // Add search term to filters array
            // Add search term to filters array
            _this.applyUniqueFilter({
                name: "searchField",
                value: term,
                operator: "="
            });
            // Reapply filters
            // Reapply filters
            _this.applyFilters();
        });
        // An event to change all filters.
        this.filtersEmitter.subscribe(function (filters) {
            // Set all emitter filters to filter array
            // Set all emitter filters to filter array
            _this.filters = filters;
            // Reapply filters
            // Reapply filters
            _this.applyFilters();
        });
        this.filterEmitter.subscribe(function (filter) {
            // Push emitted filter to filter array
            // Push emitted filter to filter array
            _this.applyUniqueFilter(filter);
            // Reapply filters
            // Reapply filters
            _this.applyFilters();
        });
        // An event to clear all filters.
        this.clearFiltersEmitter.subscribe(function (clear) {
            _this.clearFilters(clear);
        });
        this.listPreviewVisibleEmitter.subscribe(function (hidden) {
            _this.listPreviewComponent.hidden = hidden;
            if (_this.listRowComponent) {
                _this.listRowComponent.previewHidden = hidden;
            }
            _this.previewHidden = hidden;
        });
        this.activeRowEmitter.subscribe(function (row) {
            _this.activeID = row[_this.dataService.primaryKey()];
            _this.listPreviewComponent.data = row;
        });
        this.loadMoreEmitter.subscribe(function () {
            if (_this.currentPage !== _this.pageCount) {
                _this.dataService.setParam("page", ++_this.currentPage);
                _this.applyFilters(true);
            }
        });
    };
    /**
     * Resets the page index back to zero.
     *
     * @param withRefresh whether to refresh or wait.
     */
    /**
         * Resets the page index back to zero.
         *
         * @param withRefresh whether to refresh or wait.
         */
    ListViewComponent.prototype.resetPageIndex = /**
         * Resets the page index back to zero.
         *
         * @param withRefresh whether to refresh or wait.
         */
    function (withRefresh) {
        if (withRefresh === void 0) { withRefresh = false; }
        this.currentPage = 1;
        this.dataService.setParam("page", 1);
        if (withRefresh) {
            this.refresh();
        }
    };
    /**
     * @inheritdoc
     */
    /**
         * @inheritdoc
         */
    ListViewComponent.prototype.ngAfterContentInit = /**
         * @inheritdoc
         */
    function () {
        // Sync data to child components
        this.updateSubscriptions();
    };
    /**
     * Refreshes the list.
     */
    /**
         * Refreshes the list.
         */
    ListViewComponent.prototype.refresh = /**
         * Refreshes the list.
         */
    function (hidePreview) {
        if (hidePreview === void 0) { hidePreview = true; }
        this.updating = true;
        return this.updateSubscriptions(false, hidePreview);
    };
    /**
     * Changes all filters.
     *
     * @param filters
     */
    /**
         * Changes all filters.
         *
         * @param filters
         */
    ListViewComponent.prototype.replaceFilters = /**
         * Changes all filters.
         *
         * @param filters
         */
    function (filters) {
        this.filters = filters;
        this.applyFilters();
    };
    /**
     * Resets the data source back to default with filters if defined.
     *
     * @param {Array<ApiFilterInterface>} filters
     */
    /**
         * Resets the data source back to default with filters if defined.
         *
         * @param {Array<ApiFilterInterface>} filters
         */
    ListViewComponent.prototype.resetDataSource = /**
         * Resets the data source back to default with filters if defined.
         *
         * @param {Array<ApiFilterInterface>} filters
         */
    function (filters) {
        if (filters === void 0) { filters = null; }
        if (filters !== null) {
            this.originalFilters = filters;
        }
        this.clearFilters(true);
    };
    /**
     * Makes sure that filter is unique. Updates existing if already exists.
     *
     * @param filter
     */
    /**
         * Makes sure that filter is unique. Updates existing if already exists.
         *
         * @param filter
         */
    ListViewComponent.prototype.applyUniqueFilter = /**
         * Makes sure that filter is unique. Updates existing if already exists.
         *
         * @param filter
         */
    function (filter) {
        var item = this.filters.findIndex(function (item) { return item.name === filter.name; });
        if (item === -1) {
            this.filters.push(filter);
        }
        else {
            this.filters[item] = filter;
        }
    };
    /**
     * Applys filters and update subscriptions.
     */
    /**
         * Applys filters and update subscriptions.
         */
    ListViewComponent.prototype.applyFilters = /**
         * Applys filters and update subscriptions.
         */
    function (append) {
        if (append === void 0) { append = false; }
        // Apply filters to data service
        this.updating = true;
        this.dataSource = this.dataService
            .setParam("page", 1)
            .setParam("filters", this.filters)
            .fetchAll();
        this.updateSubscriptions(append);
    };
    /**
     * Clears all filters stored against this list view.
     *
     * @param clear
     */
    /**
         * Clears all filters stored against this list view.
         *
         * @param clear
         */
    ListViewComponent.prototype.clearFilters = /**
         * Clears all filters stored against this list view.
         *
         * @param clear
         */
    function (clear) {
        this.currentPage = 1;
        this.dataService.setParam("page", this.currentPage);
        // Clear filters here (on the parent)
        this.filters = clear ? JSON.parse(JSON.stringify(this.originalFilters)) : this.filters;
        // Clear filters for `search` component
        if (this.searchComponent) {
            this.searchComponent.clearSearchTerm();
        }
        if (this.listFiltersComponent) {
            this.listFiltersComponent.forEach(function (component) { return component.reset(); });
        }
        // Reapply filters
        this.applyFilters();
    };
    /**
     * Syncs all data to child components
     */
    /**
         * Syncs all data to child components
         */
    ListViewComponent.prototype.updateSubscriptions = /**
         * Syncs all data to child components
         */
    function (append, hidePreview) {
        var _this = this;
        if (append === void 0) { append = false; }
        if (hidePreview === void 0) { hidePreview = true; }
        return new Promise(function (resolve, reject) {
            if (_this.contentHandle) {
                _this.contentHandle.unsubscribe();
            }
            _this.contentHandle = _this.dataSource.subscribe(function (data) {
                _this.loading = false;
                _this.updating = false;
                _this.meta = data.meta;
                if (append) {
                    _this.rows = _this.rows.concat(data.payload);
                }
                else {
                    _this.rows = data.payload;
                }
                _this.currentPage = data.meta.page;
                _this.pageCount = data.meta.pageCount;
                _this.initSearchComponent();
                _this.initListRowComponent(data, append);
                _this.initPaginationComponent(data);
                _this.initCounterComponent(data);
                _this.initNotFoundComponent(data);
                _this.initNoResultsComponent(data);
                _this.initClearFiltersComponent();
                _this.initListFiltersComponent();
                _this.initDateFiltersComponent();
                _this.initListPreviewComponent(hidePreview);
                _this.initShowMoreComponent(data);
                resolve(true);
            }, function (error) {
                _this.handleRequestErrorEmitter.emit(error);
                reject(error);
            });
        });
    };
    /**
     * Initiates SearchComponent sub component.
     */
    /**
         * Initiates SearchComponent sub component.
         */
    ListViewComponent.prototype.initSearchComponent = /**
         * Initiates SearchComponent sub component.
         */
    function () {
        // Data for `search` component
        if (this.searchComponent) {
            if (this.searchComponent.target === this || this.searchComponent.target === undefined) {
                this.searchComponent.searchTerm = this.searchTerm;
            }
        }
    };
    /**
     * Initiates ListRowComponent sub component.
     */
    /**
         * Initiates ListRowComponent sub component.
         */
    ListViewComponent.prototype.initListRowComponent = /**
         * Initiates ListRowComponent sub component.
         */
    function (data, append) {
        if (append === void 0) { append = false; }
        // Data for `list-row` component
        if (this.listRowComponent) {
            this.listRowComponent.visibleEmitter = this.listPreviewVisibleEmitter;
            this.listRowComponent.dataEmitter = this.activeRowEmitter;
            if (append) {
                this.listRowComponent.rows = this.listRowComponent.rows.concat(data.payload);
            }
            else {
                this.listRowComponent.rows = data.payload;
            }
        }
    };
    /**
     * Initiates PaginationComponent sub component.
     */
    /**
         * Initiates PaginationComponent sub component.
         */
    ListViewComponent.prototype.initPaginationComponent = /**
         * Initiates PaginationComponent sub component.
         */
    function (data) {
        var _this = this;
        var page = parseInt(data.meta.page);
        var pageCount = parseInt(data.meta.pageCount);
        // Data for `pagination` component
        if (this.paginationComponent) {
            this.paginationComponent.forEach(function (component) {
                component.changePageEmitter = _this.changePageEmitter;
                component.page = page;
                component.pageCount = pageCount;
                component.pages = component.updatePagination();
            });
        }
    };
    /**
     * Initiates CounterComponent sub component.
     */
    /**
         * Initiates CounterComponent sub component.
         */
    ListViewComponent.prototype.initCounterComponent = /**
         * Initiates CounterComponent sub component.
         */
    function (data) {
        var page = parseInt(data.meta.page);
        var perPage = parseInt(data.meta.perPage);
        var totalCount = parseInt(data.meta.totalCount);
        var start = page * perPage - perPage + 1;
        var end = page * perPage;
        // Data for `counter` component
        if (this.counterComponents) {
            this.counterComponents.forEach(function (component) {
                component.start = start;
                component.end = end > totalCount ? totalCount : end;
                component.total = totalCount;
            });
        }
    };
    /**
     * Checks if the current list has been filtered in any way.
     * @returns {boolean}
     */
    /**
         * Checks if the current list has been filtered in any way.
         * @returns {boolean}
         */
    ListViewComponent.prototype.isFiltered = /**
         * Checks if the current list has been filtered in any way.
         * @returns {boolean}
         */
    function () {
        var noFilters = JSON.stringify(this.filters) === JSON.stringify(this.originalFilters);
        return this.lastSearch.length !== 0 || !noFilters;
    };
    /**
     * Initiates NotFoundComponent sub component.
     */
    /**
         * Initiates NotFoundComponent sub component.
         */
    ListViewComponent.prototype.initNotFoundComponent = /**
         * Initiates NotFoundComponent sub component.
         */
    function (data) {
        // Data for `not-found` component
        if (this.notFoundComponent) {
            this.notFoundComponent.show = this.isFiltered() && data.payload.length === 0;
        }
    };
    /**
     * Initiates NoResultsComponent sub component.
     */
    /**
         * Initiates NoResultsComponent sub component.
         */
    ListViewComponent.prototype.initNoResultsComponent = /**
         * Initiates NoResultsComponent sub component.
         */
    function (data) {
        // Data for `no-results` component
        if (this.noResultsComponent) {
            this.noResultsComponent.show = data.payload.length === 0 && !this.isFiltered();
        }
    };
    /**
     * Initiates ClearFiltersComponent sub component.
     */
    /**
         * Initiates ClearFiltersComponent sub component.
         */
    ListViewComponent.prototype.initClearFiltersComponent = /**
         * Initiates ClearFiltersComponent sub component.
         */
    function () {
        var _this = this;
        // Data for `clear-filters` component
        if (this.clearFiltersComponent) {
            this.clearFiltersComponent.forEach(function (component) {
                component.clearFiltersEmitter = _this.clearFiltersEmitter;
            });
        }
    };
    /**
     * Initiates ListFiltersComponent sub component.
     */
    /**
         * Initiates ListFiltersComponent sub component.
         */
    ListViewComponent.prototype.initListFiltersComponent = /**
         * Initiates ListFiltersComponent sub component.
         */
    function () {
        var _this = this;
        if (this.listFiltersComponent) {
            this.listFiltersComponent.forEach(function (component) {
                component.filterEmitter = _this.filterEmitter;
            });
        }
    };
    /**
     * Initiates DateFiltersComponent sub component.
     */
    /**
         * Initiates DateFiltersComponent sub component.
         */
    ListViewComponent.prototype.initDateFiltersComponent = /**
         * Initiates DateFiltersComponent sub component.
         */
    function () {
        var _this = this;
        if (this.dateFiltersComponent) {
            this.dateFiltersComponent.forEach(function (component) {
                component.filterEmitter = _this.filterEmitter;
            });
        }
    };
    /**
     * Initiates ListPreviewComponent sub component.
     */
    /**
         * Initiates ListPreviewComponent sub component.
         */
    ListViewComponent.prototype.initListPreviewComponent = /**
         * Initiates ListPreviewComponent sub component.
         */
    function (hidePreview) {
        var _this = this;
        if (hidePreview === void 0) { hidePreview = true; }
        if (this.listPreviewComponent) {
            this.listPreviewComponent.visibleEmitter = this.listPreviewVisibleEmitter;
            this.listPreviewComponent.dataEmitter = this.activeRowEmitter;
            this.listPreviewVisibleEmitter.emit(hidePreview);
            if (this.activeID && !hidePreview) {
                // For backward compatibility.
                if (this.listRowComponent) {
                    var row = this.listRowComponent.rows.find(function (row) {
                        if (row[_this.dataService.primaryKey()] === _this.activeID) {
                            return true;
                        }
                    });
                    if (row) {
                        this.listRowComponent.setActiveRow(row);
                    }
                }
                var data = this.rows.find(function (row) {
                    if (row[_this.dataService.primaryKey()] === _this.activeID) {
                        return true;
                    }
                });
                if (data) {
                    this.setActiveRow(data);
                }
            }
        }
    };
    /**
     * Initiate Show more sub component.
     *
     * @param data
     */
    /**
         * Initiate Show more sub component.
         *
         * @param data
         */
    ListViewComponent.prototype.initShowMoreComponent = /**
         * Initiate Show more sub component.
         *
         * @param data
         */
    function (data) {
        if (this.showMoreComponent) {
            var page = parseInt(data.meta.page);
            var pageCount = parseInt(data.meta.pageCount);
            var totalCount = parseInt(data.meta.totalCount);
            this.showMoreComponent.loadMoreEmitter = this.loadMoreEmitter;
            this.showMoreComponent.moreResults = page !== pageCount && totalCount !== 0;
        }
    };
    /**
     * Toggles the visibility of the list preview component.
     */
    /**
         * Toggles the visibility of the list preview component.
         */
    ListViewComponent.prototype.togglePreviewVisibility = /**
         * Toggles the visibility of the list preview component.
         */
    function () {
        this.previewHidden = !this.previewHidden;
        this.listPreviewVisibleEmitter.emit(this.previewHidden);
    };
    /**
     * Sets the current row as active.
     *
     * @param row
     * @param {Event} event
     * @return {boolean}
     */
    /**
         * Sets the current row as active.
         *
         * @param row
         * @param {Event} event
         * @return {boolean}
         */
    ListViewComponent.prototype.setActiveRow = /**
         * Sets the current row as active.
         *
         * @param row
         * @param {Event} event
         * @return {boolean}
         */
    function (row, event) {
        if (event) {
            event.stopPropagation();
            event.preventDefault();
        }
        if (this.listPreviewComponent && this.listPreviewComponent.shouldPromptReset()) {
            if (!window.confirm("Your unsaved changes will be reset, continue?")) {
                return false;
            }
        }
        if (!this.isActiveRow(row)) {
            this.activeRow = row;
            this.activeRowEmitter.emit(row);
            if (this.isPreviewHidden()) {
                this.togglePreviewVisibility();
            }
        }
        else {
            this.togglePreviewVisibility();
        }
        return true;
    };
    /**
     * Checks if the current row is the active one.
     *
     * @param row
     * @returns {boolean}
     */
    /**
         * Checks if the current row is the active one.
         *
         * @param row
         * @returns {boolean}
         */
    ListViewComponent.prototype.isActiveRow = /**
         * Checks if the current row is the active one.
         *
         * @param row
         * @returns {boolean}
         */
    function (row) {
        return this.activeRow === row && !this.isPreviewHidden();
    };
    /**
     * Checks if the preview is hidden.
     *
     * @returns {boolean}
     */
    /**
         * Checks if the preview is hidden.
         *
         * @returns {boolean}
         */
    ListViewComponent.prototype.isPreviewHidden = /**
         * Checks if the preview is hidden.
         *
         * @returns {boolean}
         */
    function () {
        return this.previewHidden;
    };
    ListViewComponent.decorators = [
        { type: Component, args: [{
                    selector: "c-list-view",
                    template: '<ng-content></ng-content>',
                },] },
    ];
    /** @nocollapse */
    ListViewComponent.ctorParameters = function () { return []; };
    ListViewComponent.propDecorators = {
        "dataService": [{ type: Input },],
        "changePageEmitter": [{ type: Input },],
        "searchTerm": [{ type: Input },],
        "filtersEmitter": [{ type: Input },],
        "filterEmitter": [{ type: Input },],
        "clearFiltersEmitter": [{ type: Input },],
        "listPreviewVisibleEmitter": [{ type: Input },],
        "activeRowEmitter": [{ type: Input },],
        "loadMoreEmitter": [{ type: Input },],
        "pageChangeScrollTop": [{ type: Input },],
        "handleRequestErrorEmitter": [{ type: Output, args: ["handleRequestError",] },],
        "previewHidden": [{ type: Input },],
        "counterComponents": [{ type: ContentChildren, args: [CounterComponent,] },],
        "searchComponent": [{ type: ContentChild, args: [SearchComponent,] },],
        "listRowComponent": [{ type: ContentChild, args: [ListRowComponent,] },],
        "paginationComponent": [{ type: ContentChildren, args: [PaginationComponent,] },],
        "notFoundComponent": [{ type: ContentChild, args: [NotFoundComponent,] },],
        "noResultsComponent": [{ type: ContentChild, args: [NoResultsComponent,] },],
        "clearFiltersComponent": [{ type: ContentChildren, args: [ClearFiltersComponent,] },],
        "listFiltersComponent": [{ type: ContentChildren, args: [ListFilterComponent,] },],
        "dateFiltersComponent": [{ type: ContentChildren, args: [DateFilterComponent,] },],
        "listPreviewComponent": [{ type: ContentChild, args: [ListPreviewComponent,] },],
        "showMoreComponent": [{ type: ContentChild, args: [ShowMoreComponent,] },],
    };
    return ListViewComponent;
}());
export { ListViewComponent };
//# sourceMappingURL=list.view.component.js.map